/*
 * check.c
 *
 * Ullrich von Bassewitz, 21.08.1998
 */



#include <stdlib.h>

#include "error.h"
#include "check.h"



/*****************************************************************************/
/*				     Data				     */
/*****************************************************************************/



/* Predefined messages */
const char* _MsgInternalError	= "Internal error: ";
const char* _MsgAbstractCall	= "Call to abstract method";
const char* _MsgPrecondition	= "Precondition violated: ";
const char* _MsgCheckFailed	= "Check failed: ";
const char* _MsgProgramAborted	= "Program aborted: ";



static void _CheckFailed (const char* msg, const char* cond,
       	       	          int code, const char* file, unsigned line);

void (*CheckFailed) (const char* Msg, const char* Cond, int Code,
       	       	     const char* File, unsigned Line) = _CheckFailed;
/* Function pointer that is called from check if the condition code is true. */



/*****************************************************************************/
/*		   	  	     Code			   	     */
/*****************************************************************************/



static void _CheckFailed (const char* Msg, const char* Cond,
         		  int Code, const char* File, unsigned Line)
{
    /* Log the error */
    if (Code) {
	Internal ("%s%s (= %d), file `%s', line %u", Msg, Cond, Code, File, Line);
    } else {
       	Internal ("%s%s, file `%s', line %u", Msg, Cond, File, Line);
    }

    /* Use abort() to create a core for debugging */
    abort ();
}



void Check (const char* Msg, const char* Cond, int Code,
	    const char* File, unsigned Line)
/* This function is called from all check macros (see below). It checks,
 * wether the given Code is true (!= 0). If so, it calls the CheckFailed
 * vector with the given strings. If not, it simply returns.
 */
{
    if (Code != 0) {
	CheckFailed (Msg, Cond, Code, File, Line);
    }
}



