; Table stuff (pattern/name/attribute)
; Handles the manipulation of the PPU memory areas not handled elsewhere

; WriteName(byte name, word tile, byte val)
; Sets tile in name table to value

ptr1    =     $8C
                .importzp tmp1,tmp2,tmp3, ptr2
                .import popax, pushax
                .export _WriteAttrib
                .export _WritePattern
                .export _LoadPattern
                .export _LoadName
                .export _LoadAttrib
                .export _LoadBG
                .export _WriteName

_WriteName:
                jsr popax
                sta tmp1
                jsr popax
                sta tmp2        ;Tile low val
                stx tmp3        ;Tile high val
                jsr popax
                asl a           ;Multiply by 4
                asl a           ;Set high address offset (0,400,800,C00)
                clc
                adc #$20
                clc
                adc tmp3
                tax
                ldx tmp2
                stx $2006
                sta $2006
                lda tmp1
                sta $2007
                rts

; WriteAttribtile(byte name, byte attr, byte val)
; Write val to attrib offset in name table

_WriteAttrib:
                jsr popax
                sta tmp1
                jsr popax
                sta tmp2        ;Save attrib offset
                jsr popax
                asl a           ;Multiply by 4
                asl a           ;Set high address offset (0,400,800,C00)
                clc
                adc #3
                tax
                lda #$C0        ;Starts at 2xC0 + offset
                clc
                adc tmp2
                stx $2006
                sta $2006
                lda tmp1
                sta $2007
                rts

; WritePattern(word addr, char *tile)
; Writes a tile to the pattern table (8x8)

_WritePattern:
                jsr popax
                sta ptr1                ;Save pointer
                stx ptr2                ;Low/High
                jsr popax
                and #$FC                ;Mask off low 2 bits for alignment
                stx $2006
                sta $2006
                ldy #0
                ldx #$F
PatternLoop:    lda (ptr1),y
                sta $2007
                iny
                dex
                bne PatternLoop
                rts

; LoadPattern(byte table, char *offset)
; Loads an entire pattern table (4kb)

_LoadPattern:
                jsr popax
                sta ptr1        ;save low
                stx ptr2        ;save high
                jsr popax
                and #1          ;One bit var (ignore everything else)
                bne LP1
LP0:            lda #0          ;Pattern table 0 ($0000)
                sta $2006
                sta $2006
                beq LP
LP1:            lda #$10        ;Pattern table 1 ($1000)
                ldx #0
                sta $2006
                stx $2006

LP:             ldy #0          ;start 4kb copy loop ($1000 bytes= 0-FFF)
                ldx #$F
                stx tmp1        ;High byte counter (0F-- + --FF)
                ldx #$FF
LPloop:         lda (ptr1),y    ;Get byte from memory
                sta $2007       ;Write to PPU
                iny             ;Increment address offset counter
                dex             ;Decrement loop counter
                bne LPloop      ;Keep looping till $100 bytes written
                ldx tmp1        ;Outside loop, get counter in x
                beq LPdone      ;If 0, finished whole loop
                dex             ;Otherwise, decrement outer loop counter
                stx tmp1        ;Put back in memory
                ldx ptr2        ;Get memory pointer in x
                inx             ;Increment (to match outer loop counter)
                stx ptr2        ;Put back in memory        
                ldx #$FF        ;Reset inner loop counter
                ldy #0          ;Reset address offset counter
                beq LPloop      ;Start another inner loop

LPdone:         rts             ;Finished

; LoadName(byte name, char *offset)
; Loads one entire name table

_LoadName:
                jsr popax
                sta ptr1        ;Save pointer to mem location
                stx ptr2
                jsr popax
                and #3          ;Only bottom 3 bits necessary
                asl a
                asl a
                clc
                adc #$20        ;Calculate high byte of memory location
                ldx #0
                sta $2006
                stx $2006
                ldx #$C0
                ldy #3
                sty tmp1
                ldy #0
LNloop:         lda (ptr1), y
                sta $2007
                iny
                dex
                bne LNloop
                ldx tmp1
                beq LNdone
                dex
                stx tmp1
                ldx ptr2
                inx
                stx ptr2
                ldx #$FF
                ldy #0
                beq LNloop
LNdone:         rts

; LoadAttrib(byte attrib, char *offset)
; Loads an entire attribute table

_LoadAttrib:
                jsr popax
                lda ptr1
                ldx ptr2
                jsr popax
                and #3
                asl a
                asl a
                clc
                adc #$23
                ldx #$C0
                sta $2006
                stx $2006
                ldx #63
                ldy #0
LAloop:         lda (ptr1),y
                sta $2007
                iny
                dex
                bne LAloop
                rts

;LoadBG(byte table, char *offset)
;Loads both the name and attribute table

_LoadBG:
                jsr popax
                sta ptr1
                stx ptr2
                jsr popax
                and #3
                asl a
                asl a
                adc #$20
                ldx #0
                sta $2006
                stx $2006
                ldx #3
                stx tmp1
                ldx #$FF
                ldy #0
LBGloop:        lda (ptr1),y
                sta $2007
                iny
                dex
                bne LBGloop
                ldx tmp1
                beq LBGdone
                dex
                stx tmp1
                ldx ptr2
                inx
                stx ptr2
                ldx #$FF
                ldy #0
                beq LBGloop
LBGdone:        rts

